/**
         * @constructor links.Timeline.ItemBackground
         * @extends links.Timeline.Item
         * User defined item type "line".
         */
        var ItemBackground = function (data, options) {
            links.Timeline.Item.call(this, data, options);
        };

        ItemBackground.prototype = new links.Timeline.Item();

        ItemBackground.prototype.createDOM = function () {
            var _this = this;
            var divLine = document.createElement("DIV");
            divLine.style.position = "absolute";
            divLine.style.width = "0px";
            
            this.dom = divLine;
            this.updateDOM();

            return divLine;
        };

        ItemBackground.prototype.showDOM = function (container) {
            var dom = this.dom;
            if (!dom) {
                dom = this.createDOM();
            }

            if (dom.parentNode != container) {
                if (dom.parentNode) {
                    this.hideDOM();
                }

                container.insertBefore(dom, container.firstChild);
                this.rendered = true;
            }
        };

        ItemBackground.prototype.hideDOM = function () {
            var dom = this.dom;
            if (dom) {
                var parent = dom.parentNode;
                if (parent) {
                    parent.removeChild(dom);
                    this.rendered = false;
                }
            }
        };

        ItemBackground.prototype.updateDOM = function () {
            var divLine = this.dom;
            if (divLine) {

                // update class
                divLine.className = "timeline-event item-line";

                if (this.isCluster) {
                    links.Timeline.addClassName(divLine, 'timeline-event-cluster');
                }

                // add item specific class name when provided
                if (this.className) {
                    links.Timeline.addClassName(divLine, this.className);
                }
            }
        };

        ItemBackground.prototype.updatePosition = function (timeline) {
            var dom = this.dom;
            if (dom) {
                var left = timeline.timeToScreen(this.start),
                    axisOnTop = timeline.options.axisOnTop,
                    axisTop = timeline.size.axis.top,
                    axisHeight = timeline.size.axis.height

                dom.style.left = (left - this.lineWidth / 2) + "px";
                dom.style.top = "0px";
                dom.style.height = axisTop + "px";
            }
        };

        ItemBackground.prototype.isVisible = function (start, end) {
            if (this.cluster) {
                return false;
            }

            return (this.start > start)
                && (this.start < end);
        };

        ItemBackground.prototype.setPosition = function (left, right) {
            var dom = this.dom;
            dom.style.left = (left - this.lineWidth / 2) + "px";
        };

        ItemBackground.prototype.getRight = function (timeline) {
            return timeline.timeToScreen(this.start);
        };